# General Dex GRPC Message for Client
## Need to be refactored to use the same logic as config.py
_config_keys = [
    "id",
    "secret",
    "redirect_uris",
    "trusted_peers",
    "public",
    "name",
    "logo_url",
]


def parse_list(list_from_vault: str | list) -> list:
    """
    Natively Vault can't support lists in UI(except json view), omg Hashicorp:)
    """
    if isinstance(list_from_vault, list):
        return list_from_vault

    if isinstance(list_from_vault, str):
        return list_from_vault.split(",")
    else:
        return []


def parse_bool(bool_from_vault: bool | str) -> bool:
    if isinstance(bool_from_vault, bool):
        return bool_from_vault
    return False


def _fill_missing_keys(config: dict) -> dict:
    for key in _config_keys:
        if key in ["redirect_uris", "trusted_peers"]:
            config[key] = parse_list(config.get(key))
        config["public"] = parse_bool(config.get("public", False))
    return config


def _camel_case_to_underscore(config: dict) -> dict:
    """
    Transform camel case response from Dex GRPC
    """
    return {
        "".join(["_" + i.lower() if i.isupper() else i for i in key]).lstrip("_"): value
        for key, value in config.items()
    }


def normalize_config(
    client_config: dict, vault_secret: str | None = None
) -> dict | None:
    """
    Normalize Dex config to one standard.
    """
    if "id" not in client_config and vault_secret is not None:
        # Set Vault secret name as client Id
        client_config["id"] = vault_secret

    if {"id", "secret"}.issubset(client_config.keys()):
        config = _camel_case_to_underscore(client_config)
        return _fill_missing_keys(config)
    return None



import threading

from dexus_vault.client import run
from dexus_vault.utils.logger import logger

# Try to import dotenv, if it's installed
try:
    from dotenv import load_dotenv, find_dotenv

    load_dotenv(find_dotenv())
except ImportError:
    logger.debug(f"if you want to use .env file, please install python-dotenv")


def main():
    """
    Start a new thread to run the synchronization process.
    """
    try:
        sync_thread = threading.Thread(target=run)
        sync_thread.start()
        sync_thread.join()  # Wait for the thread to finish
    except (KeyboardInterrupt, SystemExit):
        logger.info("Interrupt received, stopping...")
    except Exception as e:
        logger.error(f"An error occurred: {e}")


if __name__ == "__main__":
    logger.info("Starting dexus_vault...")
    main()



from prometheus_client import (
    start_http_server,
    Counter,
    REGISTRY,
    PROCESS_COLLECTOR,
    PLATFORM_COLLECTOR,
)
from dexus_vault.utils.logger import logger

"""
Define the Prometheus metrics and start the metrics server.
"""

client_create_metric = Counter("client_create", "Client creation status", ["status"])
client_delete_metric = Counter("client_delete", "Client deletion status", ["status"])
vault_client_secret = Counter(
    "vault_client_secret",
    "Number of wrong client specifications in Vault",
    ["status"],
)
# SYNC_TIME = Summary("sync_time_seconds", "Time spent on synchronization")


def start_metrics_server(
    disable_internal_metrics: bool, start_server: bool, port: int
) -> None:
    """
    Start the Prometheus metrics server on the specified port.
    """
    if not disable_internal_metrics:
        # Unregister default metrics
        REGISTRY.unregister(PROCESS_COLLECTOR)
        REGISTRY.unregister(PLATFORM_COLLECTOR)
        REGISTRY.unregister(
            REGISTRY._names_to_collectors["python_gc_collections_total"]
        )
        logger.debug("Internal metrics disabled")

    if start_server:
        start_http_server(port)
        logger.info(f"Prometheus metrics server started on port {port}")



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import dexus_vault.grpc_dexidp.dexidp.api_pb2 as dexidp_dot_api__pb2


class DexStub(object):
    """Dex represents the dex gRPC service."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetClient = channel.unary_unary(
            "/api.Dex/GetClient",
            request_serializer=dexidp_dot_api__pb2.GetClientReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.GetClientResp.FromString,
        )
        self.CreateClient = channel.unary_unary(
            "/api.Dex/CreateClient",
            request_serializer=dexidp_dot_api__pb2.CreateClientReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.CreateClientResp.FromString,
        )
        self.UpdateClient = channel.unary_unary(
            "/api.Dex/UpdateClient",
            request_serializer=dexidp_dot_api__pb2.UpdateClientReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.UpdateClientResp.FromString,
        )
        self.DeleteClient = channel.unary_unary(
            "/api.Dex/DeleteClient",
            request_serializer=dexidp_dot_api__pb2.DeleteClientReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.DeleteClientResp.FromString,
        )
        self.CreatePassword = channel.unary_unary(
            "/api.Dex/CreatePassword",
            request_serializer=dexidp_dot_api__pb2.CreatePasswordReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.CreatePasswordResp.FromString,
        )
        self.UpdatePassword = channel.unary_unary(
            "/api.Dex/UpdatePassword",
            request_serializer=dexidp_dot_api__pb2.UpdatePasswordReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.UpdatePasswordResp.FromString,
        )
        self.DeletePassword = channel.unary_unary(
            "/api.Dex/DeletePassword",
            request_serializer=dexidp_dot_api__pb2.DeletePasswordReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.DeletePasswordResp.FromString,
        )
        self.ListPasswords = channel.unary_unary(
            "/api.Dex/ListPasswords",
            request_serializer=dexidp_dot_api__pb2.ListPasswordReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.ListPasswordResp.FromString,
        )
        self.GetVersion = channel.unary_unary(
            "/api.Dex/GetVersion",
            request_serializer=dexidp_dot_api__pb2.VersionReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.VersionResp.FromString,
        )
        self.ListRefresh = channel.unary_unary(
            "/api.Dex/ListRefresh",
            request_serializer=dexidp_dot_api__pb2.ListRefreshReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.ListRefreshResp.FromString,
        )
        self.RevokeRefresh = channel.unary_unary(
            "/api.Dex/RevokeRefresh",
            request_serializer=dexidp_dot_api__pb2.RevokeRefreshReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.RevokeRefreshResp.FromString,
        )
        self.VerifyPassword = channel.unary_unary(
            "/api.Dex/VerifyPassword",
            request_serializer=dexidp_dot_api__pb2.VerifyPasswordReq.SerializeToString,
            response_deserializer=dexidp_dot_api__pb2.VerifyPasswordResp.FromString,
        )


class DexServicer(object):
    """Dex represents the dex gRPC service."""

    def GetClient(self, request, context):
        """GetClient gets a client."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateClient(self, request, context):
        """CreateClient creates a client."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdateClient(self, request, context):
        """UpdateClient updates an existing client"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteClient(self, request, context):
        """DeleteClient deletes the provided client."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreatePassword(self, request, context):
        """CreatePassword creates a password."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UpdatePassword(self, request, context):
        """UpdatePassword modifies existing password."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeletePassword(self, request, context):
        """DeletePassword deletes the password."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListPasswords(self, request, context):
        """ListPassword lists all password entries."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetVersion(self, request, context):
        """GetVersion returns version information of the server."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListRefresh(self, request, context):
        """ListRefresh lists all the refresh token entries for a particular user."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RevokeRefresh(self, request, context):
        """RevokeRefresh revokes the refresh token for the provided user-client pair.

        Note that each user-client pair can have only one refresh token at a time.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def VerifyPassword(self, request, context):
        """VerifyPassword returns whether a password matches a hash for a specific email or not."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_DexServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "GetClient": grpc.unary_unary_rpc_method_handler(
            servicer.GetClient,
            request_deserializer=dexidp_dot_api__pb2.GetClientReq.FromString,
            response_serializer=dexidp_dot_api__pb2.GetClientResp.SerializeToString,
        ),
        "CreateClient": grpc.unary_unary_rpc_method_handler(
            servicer.CreateClient,
            request_deserializer=dexidp_dot_api__pb2.CreateClientReq.FromString,
            response_serializer=dexidp_dot_api__pb2.CreateClientResp.SerializeToString,
        ),
        "UpdateClient": grpc.unary_unary_rpc_method_handler(
            servicer.UpdateClient,
            request_deserializer=dexidp_dot_api__pb2.UpdateClientReq.FromString,
            response_serializer=dexidp_dot_api__pb2.UpdateClientResp.SerializeToString,
        ),
        "DeleteClient": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteClient,
            request_deserializer=dexidp_dot_api__pb2.DeleteClientReq.FromString,
            response_serializer=dexidp_dot_api__pb2.DeleteClientResp.SerializeToString,
        ),
        "CreatePassword": grpc.unary_unary_rpc_method_handler(
            servicer.CreatePassword,
            request_deserializer=dexidp_dot_api__pb2.CreatePasswordReq.FromString,
            response_serializer=dexidp_dot_api__pb2.CreatePasswordResp.SerializeToString,
        ),
        "UpdatePassword": grpc.unary_unary_rpc_method_handler(
            servicer.UpdatePassword,
            request_deserializer=dexidp_dot_api__pb2.UpdatePasswordReq.FromString,
            response_serializer=dexidp_dot_api__pb2.UpdatePasswordResp.SerializeToString,
        ),
        "DeletePassword": grpc.unary_unary_rpc_method_handler(
            servicer.DeletePassword,
            request_deserializer=dexidp_dot_api__pb2.DeletePasswordReq.FromString,
            response_serializer=dexidp_dot_api__pb2.DeletePasswordResp.SerializeToString,
        ),
        "ListPasswords": grpc.unary_unary_rpc_method_handler(
            servicer.ListPasswords,
            request_deserializer=dexidp_dot_api__pb2.ListPasswordReq.FromString,
            response_serializer=dexidp_dot_api__pb2.ListPasswordResp.SerializeToString,
        ),
        "GetVersion": grpc.unary_unary_rpc_method_handler(
            servicer.GetVersion,
            request_deserializer=dexidp_dot_api__pb2.VersionReq.FromString,
            response_serializer=dexidp_dot_api__pb2.VersionResp.SerializeToString,
        ),
        "ListRefresh": grpc.unary_unary_rpc_method_handler(
            servicer.ListRefresh,
            request_deserializer=dexidp_dot_api__pb2.ListRefreshReq.FromString,
            response_serializer=dexidp_dot_api__pb2.ListRefreshResp.SerializeToString,
        ),
        "RevokeRefresh": grpc.unary_unary_rpc_method_handler(
            servicer.RevokeRefresh,
            request_deserializer=dexidp_dot_api__pb2.RevokeRefreshReq.FromString,
            response_serializer=dexidp_dot_api__pb2.RevokeRefreshResp.SerializeToString,
        ),
        "VerifyPassword": grpc.unary_unary_rpc_method_handler(
            servicer.VerifyPassword,
            request_deserializer=dexidp_dot_api__pb2.VerifyPasswordReq.FromString,
            response_serializer=dexidp_dot_api__pb2.VerifyPasswordResp.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "api.Dex", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class Dex(object):
    """Dex represents the dex gRPC service."""

    @staticmethod
    def GetClient(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/GetClient",
            dexidp_dot_api__pb2.GetClientReq.SerializeToString,
            dexidp_dot_api__pb2.GetClientResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreateClient(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/CreateClient",
            dexidp_dot_api__pb2.CreateClientReq.SerializeToString,
            dexidp_dot_api__pb2.CreateClientResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdateClient(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/UpdateClient",
            dexidp_dot_api__pb2.UpdateClientReq.SerializeToString,
            dexidp_dot_api__pb2.UpdateClientResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteClient(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/DeleteClient",
            dexidp_dot_api__pb2.DeleteClientReq.SerializeToString,
            dexidp_dot_api__pb2.DeleteClientResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreatePassword(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/CreatePassword",
            dexidp_dot_api__pb2.CreatePasswordReq.SerializeToString,
            dexidp_dot_api__pb2.CreatePasswordResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UpdatePassword(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/UpdatePassword",
            dexidp_dot_api__pb2.UpdatePasswordReq.SerializeToString,
            dexidp_dot_api__pb2.UpdatePasswordResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeletePassword(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/DeletePassword",
            dexidp_dot_api__pb2.DeletePasswordReq.SerializeToString,
            dexidp_dot_api__pb2.DeletePasswordResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListPasswords(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/ListPasswords",
            dexidp_dot_api__pb2.ListPasswordReq.SerializeToString,
            dexidp_dot_api__pb2.ListPasswordResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetVersion(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/GetVersion",
            dexidp_dot_api__pb2.VersionReq.SerializeToString,
            dexidp_dot_api__pb2.VersionResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListRefresh(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/ListRefresh",
            dexidp_dot_api__pb2.ListRefreshReq.SerializeToString,
            dexidp_dot_api__pb2.ListRefreshResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RevokeRefresh(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/RevokeRefresh",
            dexidp_dot_api__pb2.RevokeRefreshReq.SerializeToString,
            dexidp_dot_api__pb2.RevokeRefreshResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def VerifyPassword(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/api.Dex/VerifyPassword",
            dexidp_dot_api__pb2.VerifyPasswordReq.SerializeToString,
            dexidp_dot_api__pb2.VerifyPasswordResp.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



import os
import time

from dexus_vault.src.dex_processor import DexClient
from dexus_vault.src.vault_processor import VaultClient

from dexus_vault.utils.logger import logger
from dexus_vault.utils.metrics import start_metrics_server
from dexus_vault.utils.config import (
    get_vault_config,
    get_dex_config,
    get_metrics_config,
)
from dexus_vault.utils.client_parser import normalize_config


SYNC_INTERVAL = os.getenv("SYNC_INTERVAL", 60)


def metrics_server():
    """
    Start the Prometheus metrics server.
    """
    metrics_config = get_metrics_config()
    start_metrics_server(
        metrics_config.get("INTERNAL_METRICS"),
        metrics_config.get("METRICS_ENABLE"),
        metrics_config.get("METRICS_PORT"),
    )


def sync_dex_clients(dex_client: object, vault_clients: list) -> set:
    """
    Synchronize Dex clients with Vault clients.
    """
    logger.debug(f"Target clients {[x.get('id') for x in vault_clients]}")
    for client in vault_clients:
        dex_get_client = dex_client.get_dex_client(client_id=client.get("id"))

        if dex_get_client is not None:
            client_from_dex = normalize_config(
                dex_get_client.get("client", {})
            )  # TODO: move that logic to dex_processor
            if client.get("id") == client_from_dex.get("id"):

                if client == client_from_dex:
                    logger.debug(f"Client '{client_from_dex.get('id')}' already exist.")

                else:
                    logger.info(
                        f"Detected changes in '{client_from_dex.get('id')}' client configuration, will be recreated"
                    )
                    dex_client.delete_dex_client(client_from_dex.get("id"))
                    dex_client.create_dex_client(client)
        else:
            logger.info(f"Client '{client.get('id')}' not found, will be created")
            dex_client.create_dex_client(client)


def run():
    """
    Main function to run the Dex client and Vault client synchronization.
    """

    dex_client = DexClient(config=get_dex_config())
    metrics_server()
    dex_client.dex_waiter()

    while True:

        dex_client = DexClient(config=get_dex_config())
        vault_client = VaultClient(config=get_vault_config())
        client_configs = vault_client.vault_read_secrets()

        sync_dex_clients(dex_client, client_configs)
        logger.info(f"Sync completed, next sync after {SYNC_INTERVAL} seconds")
        time.sleep(SYNC_INTERVAL)



import os
from typing import Any, Dict, Optional

from dexus_vault.utils.files import load_file
from dexus_vault.utils.types import check_var_type


def _get_config_value(
    key: str, target_type: type, default: Optional[Any] = None
) -> Any:
    """
    Get the configuration value for the given key, cast it to the specified target type,
    and return the default value if the environment variable is not set.
    """
    env_value = os.getenv(key, default)
    return check_var_type(env_value, target_type)


def get_dex_config() -> Dict[str, Any]:
    """
    Get the configuration as a dictionary, with type-checked values.
    """
    config = {
        "CLIENT_CRT": load_file(_get_config_value("CLIENT_CRT", str)),
        "CLIENT_KEY": load_file(_get_config_value("CLIENT_KEY", str)),
        "CA_CRT": load_file(_get_config_value("CA_CRT", str)),
        "DEX_GRPC_URL": _get_config_value("DEX_GRPC_URL", str, "127.0.0.1:5557"),
        "DEX_MAX_RETRIES": _get_config_value("DEX_MAX_RETRIES", int, 20),
        "DEX_RETRY_WAIT": _get_config_value("DEX_RETRY_WAIT", int, 3),
    }
    return config


def get_metrics_config() -> Dict[str, Any]:
    """
    Get the configuration as a dictionary, with type-checked values.
    """
    return {
        "METRICS_ENABLE": _get_config_value("METRICS_ENABLE", bool, True),
        "METRICS_PORT": _get_config_value("METRICS_PORT", int, 8000),
        "INTERNAL_METRICS": _get_config_value("INTERNAL_METRICS", bool, False),
    }


def get_vault_config() -> Dict[str, Any]:
    """
    Get the configuration as a dictionary, with type-checked values.
    """
    # TODO: Check if it really need assignment
    config = {
        "VAULT_ADDR": _get_config_value("VAULT_ADDR", str, "http://127.0.0.1:8200"),
        "VAULT_APPROLE": _get_config_value("VAULT_APPROLE", str),
        "VAULT_APPROLE_ROLE_ID": _get_config_value("VAULT_APPROLE_ROLE_ID", str),
        "VAULT_APPROLE_SECRET_ID": _get_config_value("VAULT_APPROLE_SECRET_ID", str),
        "VAULT_APPROLE_PATH": _get_config_value("VAULT_APPROLE_PATH", str),
        "VAULT_TOKEN": _get_config_value("VAULT_TOKEN", str),
        "VAULT_CERT": _get_config_value("VAULT_CERT", str),
        "VAULT_CERT_KEY": _get_config_value("VAULT_CERT_KEY", str),
        "VAULT_CERT_CA": _get_config_value("VAULT_CERT_CA", bool | str, False),
        "VAULT_LDAP_USERNAME": _get_config_value("VAULT_LDAP_USERNAME", str),
        "VAULT_LDAP_PASSWORD": _get_config_value("VAULT_LDAP_PASSWORD", str),
        "VAULT_REQUEST_TIMEOUT": _get_config_value("VAULT_REQUEST_TIMEOUT", int, 5),
        "VAULT_MAX_RETRIES": _get_config_value("VAULT_MAX_RETRIES", int, 20),
        "VAULT_RETRY_WAIT": _get_config_value("VAULT_RETRY_WAIT", int, 3),
        "VAULT_ALLOW_REDIRECT": _get_config_value("VAULT_ALLOW_REDIRECT", bool, False),
        "VAULT_NAMESPACE": _get_config_value("VAULT_NAMESPACE", str),
        "VAULT_PROXIES": _get_config_value("VAULT_PROXIES", dict),
        "VAULT_MOUNT_POINT": _get_config_value("VAULT_MOUNT_POINT", str),
        "VAULT_CLIENTS_PATH": _get_config_value("VAULT_CLIENTS_PATH", str),
        "VAULT_ENGINE": _get_config_value("VAULT_ENGINE", str, "v2"),
    }
    return config



import os


def load_file(filepath: str) -> bytes:
    """
    Loads file from filepath(preferred absolute path)
    """
    if filepath is not None:
        real_path = os.path.join(os.getcwd(), filepath)
        try:
            with open(real_path, "rb") as f:
                return f.read()

        except FileNotFoundError:
            raise FileNotFoundError(
                f"File {filepath} path is incorrect or file does not exist!"
            )
    return None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dexidp/api.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x10\x64\x65xidp/api.proto\x12\x03\x61pi"\x82\x01\n\x06\x43lient\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06secret\x18\x02 \x01(\t\x12\x15\n\rredirect_uris\x18\x03 \x03(\t\x12\x15\n\rtrusted_peers\x18\x04 \x03(\t\x12\x0e\n\x06public\x18\x05 \x01(\x08\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\x10\n\x08logo_url\x18\x07 \x01(\t"\x1a\n\x0cGetClientReq\x12\n\n\x02id\x18\x01 \x01(\t",\n\rGetClientResp\x12\x1b\n\x06\x63lient\x18\x01 \x01(\x0b\x32\x0b.api.Client".\n\x0f\x43reateClientReq\x12\x1b\n\x06\x63lient\x18\x01 \x01(\x0b\x32\x0b.api.Client"G\n\x10\x43reateClientResp\x12\x16\n\x0e\x61lready_exists\x18\x01 \x01(\x08\x12\x1b\n\x06\x63lient\x18\x02 \x01(\x0b\x32\x0b.api.Client"\x1d\n\x0f\x44\x65leteClientReq\x12\n\n\x02id\x18\x01 \x01(\t"%\n\x10\x44\x65leteClientResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08"k\n\x0fUpdateClientReq\x12\n\n\x02id\x18\x01 \x01(\t\x12\x15\n\rredirect_uris\x18\x02 \x03(\t\x12\x15\n\rtrusted_peers\x18\x03 \x03(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x10\n\x08logo_url\x18\x05 \x01(\t"%\n\x10UpdateClientResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08"J\n\x08Password\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x0c\n\x04hash\x18\x02 \x01(\x0c\x12\x10\n\x08username\x18\x03 \x01(\t\x12\x0f\n\x07user_id\x18\x04 \x01(\t"4\n\x11\x43reatePasswordReq\x12\x1f\n\x08password\x18\x01 \x01(\x0b\x32\r.api.Password",\n\x12\x43reatePasswordResp\x12\x16\n\x0e\x61lready_exists\x18\x01 \x01(\x08"J\n\x11UpdatePasswordReq\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x10\n\x08new_hash\x18\x02 \x01(\x0c\x12\x14\n\x0cnew_username\x18\x03 \x01(\t"\'\n\x12UpdatePasswordResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08""\n\x11\x44\x65letePasswordReq\x12\r\n\x05\x65mail\x18\x01 \x01(\t"\'\n\x12\x44\x65letePasswordResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08"\x11\n\x0fListPasswordReq"4\n\x10ListPasswordResp\x12 \n\tpasswords\x18\x01 \x03(\x0b\x32\r.api.Password"\x0c\n\nVersionReq"*\n\x0bVersionResp\x12\x0e\n\x06server\x18\x01 \x01(\t\x12\x0b\n\x03\x61pi\x18\x02 \x01(\x05"W\n\x0fRefreshTokenRef\x12\n\n\x02id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t\x12\x12\n\ncreated_at\x18\x05 \x01(\x03\x12\x11\n\tlast_used\x18\x06 \x01(\x03"!\n\x0eListRefreshReq\x12\x0f\n\x07user_id\x18\x01 \x01(\t"?\n\x0fListRefreshResp\x12,\n\x0erefresh_tokens\x18\x01 \x03(\x0b\x32\x14.api.RefreshTokenRef"6\n\x10RevokeRefreshReq\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t"&\n\x11RevokeRefreshResp\x12\x11\n\tnot_found\x18\x01 \x01(\x08"4\n\x11VerifyPasswordReq\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t"9\n\x12VerifyPasswordResp\x12\x10\n\x08verified\x18\x01 \x01(\x08\x12\x11\n\tnot_found\x18\x02 \x01(\x08\x32\xfd\x05\n\x03\x44\x65x\x12\x34\n\tGetClient\x12\x11.api.GetClientReq\x1a\x12.api.GetClientResp"\x00\x12=\n\x0c\x43reateClient\x12\x14.api.CreateClientReq\x1a\x15.api.CreateClientResp"\x00\x12=\n\x0cUpdateClient\x12\x14.api.UpdateClientReq\x1a\x15.api.UpdateClientResp"\x00\x12=\n\x0c\x44\x65leteClient\x12\x14.api.DeleteClientReq\x1a\x15.api.DeleteClientResp"\x00\x12\x43\n\x0e\x43reatePassword\x12\x16.api.CreatePasswordReq\x1a\x17.api.CreatePasswordResp"\x00\x12\x43\n\x0eUpdatePassword\x12\x16.api.UpdatePasswordReq\x1a\x17.api.UpdatePasswordResp"\x00\x12\x43\n\x0e\x44\x65letePassword\x12\x16.api.DeletePasswordReq\x1a\x17.api.DeletePasswordResp"\x00\x12>\n\rListPasswords\x12\x14.api.ListPasswordReq\x1a\x15.api.ListPasswordResp"\x00\x12\x31\n\nGetVersion\x12\x0f.api.VersionReq\x1a\x10.api.VersionResp"\x00\x12:\n\x0bListRefresh\x12\x13.api.ListRefreshReq\x1a\x14.api.ListRefreshResp"\x00\x12@\n\rRevokeRefresh\x12\x15.api.RevokeRefreshReq\x1a\x16.api.RevokeRefreshResp"\x00\x12\x43\n\x0eVerifyPassword\x12\x16.api.VerifyPasswordReq\x1a\x17.api.VerifyPasswordResp"\x00\x42\x36\n\x12\x63om.coreos.dex.apiZ github.com/dexidp/dex/api/v2;apib\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "dexidp.api_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals["DESCRIPTOR"]._serialized_options = (
        b"\n\022com.coreos.dex.apiZ github.com/dexidp/dex/api/v2;api"
    )
    _globals["_CLIENT"]._serialized_start = 26
    _globals["_CLIENT"]._serialized_end = 156
    _globals["_GETCLIENTREQ"]._serialized_start = 158
    _globals["_GETCLIENTREQ"]._serialized_end = 184
    _globals["_GETCLIENTRESP"]._serialized_start = 186
    _globals["_GETCLIENTRESP"]._serialized_end = 230
    _globals["_CREATECLIENTREQ"]._serialized_start = 232
    _globals["_CREATECLIENTREQ"]._serialized_end = 278
    _globals["_CREATECLIENTRESP"]._serialized_start = 280
    _globals["_CREATECLIENTRESP"]._serialized_end = 351
    _globals["_DELETECLIENTREQ"]._serialized_start = 353
    _globals["_DELETECLIENTREQ"]._serialized_end = 382
    _globals["_DELETECLIENTRESP"]._serialized_start = 384
    _globals["_DELETECLIENTRESP"]._serialized_end = 421
    _globals["_UPDATECLIENTREQ"]._serialized_start = 423
    _globals["_UPDATECLIENTREQ"]._serialized_end = 530
    _globals["_UPDATECLIENTRESP"]._serialized_start = 532
    _globals["_UPDATECLIENTRESP"]._serialized_end = 569
    _globals["_PASSWORD"]._serialized_start = 571
    _globals["_PASSWORD"]._serialized_end = 645
    _globals["_CREATEPASSWORDREQ"]._serialized_start = 647
    _globals["_CREATEPASSWORDREQ"]._serialized_end = 699
    _globals["_CREATEPASSWORDRESP"]._serialized_start = 701
    _globals["_CREATEPASSWORDRESP"]._serialized_end = 745
    _globals["_UPDATEPASSWORDREQ"]._serialized_start = 747
    _globals["_UPDATEPASSWORDREQ"]._serialized_end = 821
    _globals["_UPDATEPASSWORDRESP"]._serialized_start = 823
    _globals["_UPDATEPASSWORDRESP"]._serialized_end = 862
    _globals["_DELETEPASSWORDREQ"]._serialized_start = 864
    _globals["_DELETEPASSWORDREQ"]._serialized_end = 898
    _globals["_DELETEPASSWORDRESP"]._serialized_start = 900
    _globals["_DELETEPASSWORDRESP"]._serialized_end = 939
    _globals["_LISTPASSWORDREQ"]._serialized_start = 941
    _globals["_LISTPASSWORDREQ"]._serialized_end = 958
    _globals["_LISTPASSWORDRESP"]._serialized_start = 960
    _globals["_LISTPASSWORDRESP"]._serialized_end = 1012
    _globals["_VERSIONREQ"]._serialized_start = 1014
    _globals["_VERSIONREQ"]._serialized_end = 1026
    _globals["_VERSIONRESP"]._serialized_start = 1028
    _globals["_VERSIONRESP"]._serialized_end = 1070
    _globals["_REFRESHTOKENREF"]._serialized_start = 1072
    _globals["_REFRESHTOKENREF"]._serialized_end = 1159
    _globals["_LISTREFRESHREQ"]._serialized_start = 1161
    _globals["_LISTREFRESHREQ"]._serialized_end = 1194
    _globals["_LISTREFRESHRESP"]._serialized_start = 1196
    _globals["_LISTREFRESHRESP"]._serialized_end = 1259
    _globals["_REVOKEREFRESHREQ"]._serialized_start = 1261
    _globals["_REVOKEREFRESHREQ"]._serialized_end = 1315
    _globals["_REVOKEREFRESHRESP"]._serialized_start = 1317
    _globals["_REVOKEREFRESHRESP"]._serialized_end = 1355
    _globals["_VERIFYPASSWORDREQ"]._serialized_start = 1357
    _globals["_VERIFYPASSWORDREQ"]._serialized_end = 1409
    _globals["_VERIFYPASSWORDRESP"]._serialized_start = 1411
    _globals["_VERIFYPASSWORDRESP"]._serialized_end = 1468
    _globals["_DEX"]._serialized_start = 1471
    _globals["_DEX"]._serialized_end = 2236
# @@protoc_insertion_point(module_scope)



import os
from codecs import open

from setuptools import setup, find_packages

package_name = "dexus_vault"
here = os.path.abspath(os.path.dirname(__file__))

# Read README.md for docs
with open("README.md", "r", "utf-8") as f:
    long_description = f.read()

# Read __version__.py for version
about = {}
with open(os.path.join(here, package_name, "__version__.py"), "r", "utf-8") as f:
    exec(f.read(), about)

setup(
    name=about["__title__"],
    version=about["__version__"],
    author="ifurs",
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=about["__url__"],
    license=about["__license__"],
    packages=find_packages(),
    classifiers=(
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: CPython",
        "Operating System :: OS Independent",
    ),
    install_requires=[
        "grpcio~=1.62.0",
        "grpcio-tools~=1.60.1",
        "googleapis-common-protos>=1.62.0",
        "protobuf~=4.25.3",
        "hvac~=2.1.0",
        "prometheus_client~=0.20.0",
    ],
    python_requires=">=3.8",
    project_urls={"Source": about["__url__"]},
    entry_points={
        "console_scripts": [f"{about['__title__']} = dexus_vault.__main__:main"]
    },
)



import time
import grpc

# Import function that transform GRPC Message into Python dict
from google.protobuf.json_format import MessageToDict

# Load GRPC Stub and Methods from pb2 defined using protobuf builder
from dexus_vault.grpc_dexidp.dexidp.api_pb2_grpc import DexStub
import dexus_vault.grpc_dexidp.dexidp.api_pb2 as pb2

from dexus_vault.utils.logger import logger
from dexus_vault.utils.metrics import client_create_metric, client_delete_metric


class DexClient:
    """
    This class defines all logic for operating with Dex gRPC API
    """

    def __init__(self, config):
        # credentials used for Dex connection including GRPC URL
        self.config = config

        # channel credentials obj placeholder
        self.channel = self.dex_grpc_connect()

    # crete grpc connection to Dex
    def dex_grpc_connect(self) -> object:
        """
        Open connection to Dex gRPC
        """

        if self.config["CLIENT_CRT"]:
            self.creds = grpc.ssl_channel_credentials(
                root_certificates=self.config["CA_CRT"],
                private_key=self.config["CLIENT_KEY"],
                certificate_chain=self.config["CLIENT_CRT"],
            )
            return grpc.secure_channel(
                target=self.config["DEX_GRPC_URL"], credentials=self.creds
            )
        else:
            logger.debug(f"Dex client started in insecure channel")
            return grpc.insecure_channel(target=self.config["DEX_GRPC_URL"])

    def dex_grpc_close_connection(self):
        """
        Close connection to Dex gRPC
        """
        if self.channel:
            self.channel.close()

    def dex_waiter(self) -> bool:
        """
        Ensure connection to Dex gRPC
        """
        _retry = 0
        while True:
            try:
                logger.info(f"Dex server version {self.get_dex_version()}")
                return True
            except Exception as error:
                if _retry >= self.config["DEX_MAX_RETRIES"]:
                    raise RuntimeError(f"Could not connect to Dex gRPC server: {error}")
                else:
                    _retry += 1
                    logger.debug(f"Dex gRPC is unavailable, retying...")
                    time.sleep(self.config["DEX_RETRY_WAIT"])

    def get_dex_version(self) -> dict:
        """
        Get Dex version
        """
        dex_request = pb2.VersionReq()
        response = DexStub(self.channel).GetVersion(dex_request)

        return MessageToDict(response)

    def get_dex_client(self, client_id: str) -> dict | None:
        """
        Call Get Dex Client with by client id
        """
        dex_request = pb2.GetClientReq()
        dex_request.id = client_id

        try:
            response = DexStub(self.channel).GetClient(dex_request)
            return MessageToDict(response)

        except grpc.RpcError as rpc_error:
            if rpc_error.code() == grpc.StatusCode.UNKNOWN:
                logger.debug(
                    f"RESPONSE FROM GRPC: {rpc_error.details()}, client {client_id}"
                )
            else:
                logger.warning(
                    f"GRPC CALL CODE: {rpc_error.code()} with details {rpc_error.details()}"
                )

    def create_dex_client(self, client: dict) -> dict | None:
        """
        Create OIDC client in Dex
        """
        try:
            request = pb2.CreateClientReq()
            request.client.id = client.get("id")
            request.client.secret = client.get("secret")
            request.client.redirect_uris.extend(client.get("redirect_uris", ""))
            request.client.trusted_peers.extend(client.get("trusted_peers", ""))
            request.client.public = client.get("public", 0)
            request.client.name = client.get("name", "")
            request.client.logo_url = client.get("logo_url", "")

            response = MessageToDict(DexStub(self.channel).CreateClient(request))
            if response.get("client", None) is not None:
                client_id = response.get("client").get("id")
                client_create_metric.labels(status="ok").inc()
                logger.info(f"Created new Dex client '{client_id}'")
                return client_id
            else:
                logger.warning(f"RESPONSE FROM GRPC: {response}")

        except Exception as error:
            client_create_metric.labels(status="failed").inc()
            logger.error(f"Failed to create client {client.get('id')}")
            logger.error(f"RESPONSE FROM GRPC: {error}")

    def delete_dex_client(self, client_id: str) -> None:
        """
        Delete OIDC client in Dex
        """
        try:
            dex_request = pb2.DeleteClientReq()
            dex_request.id = client_id

            # Because of Dex implementation, this request returns None
            # Or simple dict {'notFound': True} so it need to be processed
            response = MessageToDict(DexStub(self.channel).DeleteClient(dex_request))

            if response.get("notFound", None) is not None:
                client_delete_metric.labels(status="failed").inc()
                logger.warning(f"Client '{client_id}' not found")
            else:
                client_create_metric.labels(status="ok").inc()
                logger.info(f"client {client_id} was deleted")
        except Exception as error:
            client_delete_metric.labels(status="failed").inc()
            logger.error(f"Failed to delete client {client_id}")
            logger.error(f"RESPONSE FROM GRPC: {error}")

    def __del__(self):
        """
        Every time DexClient object deletes, this will close gRPC connection
        """
        self.dex_grpc_close_connection()
        logger.debug(f"Dex connection closed")



from typing import Any, TypeVar, Type

T = TypeVar("T")


def check_var_type(value: Any, target_type: Type[T]) -> T:
    """
    Cast the given value to the specified target type.
    """
    try:
        if value and isinstance(value, target_type):
            return value
    except (ValueError, TypeError):
        # If the cast fails, return the original value
        return None



import sys
import time
import requests
import hvac

from dexus_vault.utils.client_parser import normalize_config
from dexus_vault.utils.logger import logger
from dexus_vault.utils.metrics import vault_client_secret


class VaultClient:
    """
    This Class represents methods we used inside dexus_vault, nothing new, just wrap hvac
    """

    def __init__(self, config: dict):
        self.config = config
        self.client = self.login_to_client()

    def _check_vault_status(self, client: object) -> None:
        """
        Function validates if Vault is up and running
        """

        for _ in range(self.config["VAULT_MAX_RETRIES"]):
            try:
                response = client.sys.read_health_status(method="GET")
                if isinstance(response, requests.Response):
                    status = response.json()
                else:
                    status = response
                if status["initialized"]:
                    logger.debug(f"Vault {self.config['VAULT_ADDR']} is initialized")
                    return True
                else:
                    logger.warning(
                        f"Vault {self.config['VAULT_ADDR']} is not initialized {status}"
                    )
            except requests.exceptions.ConnectionError:
                logger.warning(f"Vault {self.config['VAULT_ADDR']} connection failed")
            time.sleep(self.config["VAULT_RETRY_WAIT"])
        logger.error(f"Vault {self.config['VAULT_ADDR']} unreachable, exiting...")
        sys.exit(1)

    def _check_if_vault_auth(self, client: object, auth_method: str) -> None:
        """
        Function validates if we successfully authenticated to Vault
        """

        if client.is_authenticated():
            logger.debug(
                f"Authenticated to Vault {self.config['VAULT_ADDR']} via {auth_method} auth method"
            )

        else:
            raise RuntimeError(
                f"Failed to authenticate to Vault {self.config['VAULT_ADDR']} via {auth_method} auth method "
            )

    def login_to_client(self) -> object:
        """
        Define auth method for Vault and authenticate
        """
        client = hvac.Client(url=self.config["VAULT_ADDR"])
        self._check_vault_status(client)

        if self.config["VAULT_APPROLE"] is not None:
            auth_method = "approle"
            client.sys.enable_auth_method(
                method_type=self.auth_method,
            )

            if self.config["VAULT_APPROLE_ROLE_ID"]:
                client.auth.approle.login(
                    role_id=self.config["VAULT_APPROLE_ROLE_ID"],
                    secret_id=self.config["VAULT_APPROLE_SECRET_ID"],
                )

            elif self.config["VAULT_APPROLE_PATH"]:
                client.sys.enable_auth_method(
                    method_type=self.auth_method,
                    path=self.config["VAULT_APPROLE_PATH"],
                )

        elif self.config["VAULT_LDAP_USERNAME"] is not None:
            auth_method = "LDAP"
            client.auth.ldap.login(
                username=self.config["VAULT_LDAP_USERNAME"],
                password=self.config["VAULT_LDAP_PASSWORD"],
            )

        elif self.config["VAULT_CERT"] is not None:
            auth_method = "Client cert"
            client = hvac.Client(
                url=self.config["VAULT_ADDR"],
                token=self.config["VAULT_TOKEN"],
                cert=(self.config["VAULT_CERT"], self.config["VAULT_CERT_KEY"]),
                verify=(self.config["VAULT_CERT_CA"]),
            )

        elif self.config["VAULT_TOKEN"] is not None:
            auth_method = "Vault token"
            client.token = self.config["VAULT_TOKEN"]

        else:
            raise KeyError(f"Vault auth method is not specified!")

        self._check_if_vault_auth(client, auth_method)
        return client

    def vault_list_secrets(self) -> list:
        """
        List secrets from Vault by path
        """

        if self.config["VAULT_ENGINE"] == "v1":
            response = self.client.secrets.kv.v1.list_secrets(
                self.config["VAULT_CLIENTS_PATH"],
                mount_point=self.config["VAULT_MOUNT_POINT"],
            )

        elif self.config["VAULT_ENGINE"] == "v2":
            response = self.client.secrets.kv.v2.list_secrets(
                self.config["VAULT_CLIENTS_PATH"],
                mount_point=self.config["VAULT_MOUNT_POINT"],
            )

        return response["data"]["keys"]

    def vault_read_secret(self, secret_path: str):
        """
        Read specified secret from Vault
        """

        if self.config["VAULT_ENGINE"] == "v1":
            response = self.client.secrets.kv.v1.read_secret(
                path=f"{self.config['VAULT_CLIENTS_PATH']}/{secret_path}",
                mount_point=self.config["VAULT_MOUNT_POINT"],
            )
            return response["data"]

        elif self.config["VAULT_ENGINE"] == "v2":
            response = self.client.secrets.kv.read_secret_version(
                path=f"{self.config['VAULT_CLIENTS_PATH']}/{secret_path}",
                mount_point=self.config["VAULT_MOUNT_POINT"],
            )
            return response["data"]["data"]
        return None

    def vault_read_secrets(self) -> list:
        """
        Combine list_secrets with read_secret
        """
        _client_config = []

        for secret in self.vault_list_secrets():
            config = normalize_config(
                self.vault_read_secret(secret_path=secret), secret
            )

            if config is not None:
                _client_config.append(config)
                vault_client_secret.labels(status="ok").inc()

            else:
                logger.warning(
                    f"Secret '{secret}' in Vault, missing 'secret' key, or have incorrect structure"
                )
                vault_client_secret.labels(status="failed").inc()

        return _client_config



import os
import sys
import logging


log_level = os.getenv("LOG_LEVEL", "INFO")
# Define a format for your logs
log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

# Create a stream handler
stream_handler = logging.StreamHandler(sys.stdout)
stream_handler.setLevel(log_level)
stream_handler.setFormatter(logging.Formatter(log_format))

## Create a file handler
# file_handler = logging.FileHandler('dexus_vault.log')
# file_handler.setLevel(log_level)
# file_handler.setFormatter(logging.Formatter(log_format))

# Get a logger
logger = logging.getLogger("dexus_vault")
logger.setLevel(log_level)

# Add the handlers to the logger
logger.addHandler(stream_handler)
# logger.addHandler(file_handler)

# Set log level for external libraries
logging.getLogger("httpx").setLevel(logging.WARNING)
